function gui_shock_decomposition(tabId)
% function gui_shock_decomposition(tabId)
% interface for the DYNARE shock_decomposition command
%
% INPUTS
%   tabId:  GUI tab element which displays shock_decomposition command interface
%
% OUTPUTS
%   none
%
% SPECIAL REQUIREMENTS
%   none

% Copyright (C) 2019 Dynare Team
%
% This file is part of Dynare.
%
% Dynare is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% Dynare is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with Dynare.  If not, see <http://www.gnu.org/licenses/>.

global project_info model_settings
global M_ oo_ options_ bayestopt_ estim_params_

bg_color = char(getappdata(0,'bg_color'));
special_color = char(getappdata(0,'special_color'));

handles = [];
gui_size = gui_tools.get_gui_elements_size(tabId);

first_period = project_info.first_obs_date;
if ~isnan(options_.first_obs)
    first_period = first_period + options_.first_obs - 1;
end
last_period = project_info.last_obs_date;
if ~isnan(options_.nobs)
    last_period = first_period+options_.nobs-1;
end

% Panels
handles.uipanelVars = uipanel( ...
    'Parent', tabId, ...
    'Tag', 'uipanelVars', ...
    'BackgroundColor', special_color, ...
    'Units', 'normalized', ...
    'Position', [0.01 0.09 0.48 0.82], ...
    'Title', '', ...
    'BorderType', 'none');

handles = gui_tabs.create_endo_vars(handles, 'shock_decomposition');

handles.uipanelResults = uipanel( ...
    'Parent', tabId, ...
    'Tag', 'uipanelVars', ...
    'BackgroundColor', bg_color,...
    'Units', 'normalized', ...
    'Position', [0.51 0.09 0.48 0.82], ...
    'Title', 'Command and results options:');

uipanelResults_CreateFcn();

% Text
uicontrol( ...
    'Parent', tabId, ...
    'Style', 'text', ...
    'BackgroundColor', bg_color,...
    'Units', 'normalized', ...
    'Position', [0.01 0.92 0.48 0.05], ...
    'FontWeight', 'bold', ...
    'String', 'Select endogenous variables for shock decomposition:', ...
    'HorizontalAlignment', 'left');

% Buttons
handles.pushbuttonShockDecomposition = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonSimulation', ...
    'Style', 'pushbutton', ...
    'Units', 'normalized', ....
    'Position', [gui_size.space gui_size.bottom gui_size.button_width gui_size.button_height], ...
    'String', 'Run', ...
    'Callback', @pushbuttonShockDecomposition_Callback);

handles.pushbuttonReset = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonReset', ...
    'Style', 'pushbutton', ...
    'Units', 'normalized', ...
    'Position', [gui_size.space*2+gui_size.button_width gui_size.bottom gui_size.button_width gui_size.button_height], ...
    'String', 'Reset', ...
    'Callback', @pushbuttonReset_Callback);

handles.pushbuttonCloseAll = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonSimulation', ...
    'Style', 'pushbutton', ...
    'Units', 'normalized', ...
    'Position', [gui_size.space*3+gui_size.button_width*2 gui_size.bottom gui_size.button_width gui_size.button_height], ...
    'String', 'Close all output figures', ...
    'Enable', 'on', ...
    'Callback', @pushbuttonCloseAll_Callback);

handles.pushbuttonClose = uicontrol( ...
    'Parent', tabId, ...
    'Tag', 'pushbuttonReset', ...
    'Units', 'normalized', ...
    'Position', [gui_size.space*4+gui_size.button_width*3 gui_size.bottom gui_size.button_width gui_size.button_height], ...
    'String', 'Close this tab', ...
    'Callback',{@close_tab,tabId});

    function uipanelResults_CreateFcn()
        top = 1;
        dwidth = gui_size.default_width;
        dheight = gui_size.default_height;
        spc = gui_size.c_width;

        num = 1;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text7', ...
            'Style', 'text', ...
            'BackgroundColor', bg_color,...
            'Units', 'normalized', ...
            'Position', [spc top-num*dheight 1-spc*4 dheight/2], ...
            'FontWeight', 'bold', ...
            'String', 'Specify the parameter set to use for running the smoother:', ...
            'HorizontalAlignment', 'left');
        num = num+0.8;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text8', ...
            'Style', 'text', ...
            'Units', 'normalized', ...
            'Position', [spc*3 top-num*dheight dwidth*1.5 dheight/2], ...
            'String', 'parameter set:', ...
            'HorizontalAlignment', 'left');

        handles.parameterSet = uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'parameterSet', ...
            'Style', 'popupmenu', ...
            'Units', 'normalized', ...
            'Position', [spc*3+dwidth*1.5 top-num*dheight dwidth*2 dheight/2], ...
            'CreateFcn', @parameterSet_CreateFcn);

        num = num+1.5;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Style', 'text', ...
            'BackgroundColor', bg_color,...
            'Units', 'normalized', ...
            'Position', [spc top-num*dheight 1-spc*4 dheight/2], ...
            'FontWeight', 'bold', ...
            'String', 'Select historical observations to be displayed:', ...
            'HorizontalAlignment', 'left');
        num = num+0.8;
        handles.text8 = uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text8', ...
            'Style', 'text', ...
            'Units', 'normalized', ...
            'Position', [spc*3 top-num*dheight dwidth*3.5 dheight/2], ...
            'String', 'The first historical observation to be displayed:', ...
            'HorizontalAlignment', 'left');

        handles.firstPeriodYear = uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'firstPeriodYear', ...
            'Style', 'popupmenu', ...
            'Units', 'normalized', ...
            'Position', [spc*3+dwidth*3.5 top-num*dheight dwidth*1.3 dheight/2], ...
            'CreateFcn', @firstPeriodYear_CreateFcn);
        handles.firstPeriodQuarter = uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'firstPeriodQuarter', ...
            'Style', 'popupmenu', ...
            'Units', 'normalized', ...
            'Position', [spc*4+dwidth*4.7 top-num*dheight dwidth*1.3 dheight/2], ...
            'CreateFcn', @firstPeriodQuarter_CreateFcn);

        num = num+0.8;
        handles.text9 = uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text9', ...
            'Style', 'text', ...
            'Units', 'normalized', ...
            'Position', [spc*3 top-num*dheight dwidth*3.5 dheight/2], ...
            'String', 'The last historical observation to be displayed:', ...
            'HorizontalAlignment', 'left');

        handles.lastPeriodYear = uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'lastPeriodYear', ...
            'Style', 'popupmenu', ...
            'Units', 'normalized', ...
            'Position', [spc*3+dwidth*3.5 top-num*dheight dwidth*1.3 dheight/2], ...
            'CreateFcn', @lastPeriodYear_CreateFcn);
        handles.lastPeriodQuarter = uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'lastPeriodQuarter', ...
            'Style', 'popupmenu', ...
            'Units', 'normalized', ...
            'Position', [spc*4+dwidth*4.7 top-num*dheight dwidth*1.3 dheight/2], ...
            'CreateFcn', @lastPeriodQuarter_CreateFcn);

        num = num+1.5;
        handles.useShockGrouping = uicontrol(...
            'Parent', handles.uipanelResults, ...
            'Style','checkbox',...
            'Units', 'normalized', ...
            'Position', [spc top-num*dheight 1-spc*4 dheight/2], ...
            'String', 'Use shock groups when displaying decomposition',...
            'FontWeight', 'bold');

        num = num+1.5;
        handles.useInitState = uicontrol(...
            'Parent', handles.uipanelResults, ...
            'Style','checkbox',...
            'Units', 'normalized', ...
            'Position', [spc top-num*dheight 1-spc*4 dheight/2], ...
            'String', 'init_state (condition on smoothed states in period 1)',...
            'FontWeight', 'bold');
        
        num = num + 1;
        uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'text8', ...
            'Style', 'text', ...
            'Units', 'normalized', ...
            'Position', [spc*3 top-num*dheight dwidth*1.5 dheight/2], ...
            'String', 'type:', ...
            'HorizontalAlignment', 'left');

        handles.type = uicontrol( ...
            'Parent', handles.uipanelResults, ...
            'Tag', 'type', ...
            'Style', 'popupmenu', ...
            'Units', 'normalized', ...
            'Position', [spc*3+dwidth*1.5 top-num*dheight dwidth*2 dheight/2], ...
            'String', {'qoq','yoy','aoa'}, ...
            'Value', 1);
        
        num = num+1.5;
        handles.noGraph = uicontrol(...
            'Parent', handles.uipanelResults, ...
            'Style','checkbox',...
            'Units', 'normalized', ...
            'Position', [spc top-num*dheight 1-spc*4 dheight/2], ...
            'String', 'Only compute shock decomposition, do not plot',...
            'FontWeight', 'bold');
    end

    function pushbuttonShockDecomposition_Callback(hObject, ~)
        if strcmp(handles.parameterSet.String{handles.parameterSet.Value}, 'calibration')
            if ~project_info.stochastic_simulation_run
                gui_tools.show_warning('To run shock decomposition with the `calibration` parameter set, you must first run stochastic simulation');
                uicontrol(hObject);
                return
            end
            if ~project_info.observed_variables_set
                gui_tools.show_warning('To run shock decomposition with the `calibration` parameter set, you must first go to Estimation->Observed variables & data file');
                uicontrol(hObject);
                return
            end
        else
            if ~project_info.estimation_run
                gui_tools.show_warning('To run shock decomposition you must first run stochastic simulation or estimation');
                uicontrol(hObject);
                return
            end
        end
        old_options = options_;
        options_.parameter_set = handles.parameterSet.String{handles.parameterSet.Value};
        options_.datafile = project_info.data_file;
        options_.nodisplay = 0;
        options_.plot_priors = 0;
        options_.type = handles.type.String;
        gui_tools.project_log_entry('Running shock decomposition', '...');
        [jObj, guiObj] = gui_tools.create_animated_screen('Running shock decomposition, please wait...', tabId);
        var_list_ = handles.endoTable.Data([handles.endoTable.Data{:,1}], 2);
        dynare_default = true;
        if isfield(oo_, 'FilteredVariables') && isfield(oo_, 'SmoothedVariables') && isfield(oo_, 'UpdatedVariables')
            if isfield(oo_.FilteredVariables, 'Mean')
                dynare_default = false;
            end
        end

        % computations take place here
        try
            if ~dynare_default
                T0 = (handles.firstPeriodYear.Value - 1) * 4 + handles.firstPeriodQuarter.Value - handles.firstPeriodQuarterDefault + 1;
                T1 = (handles.lastPeriodYear.Value - 1) * 4 + handles.lastPeriodQuarter.Value - handles.firstPeriodQuarterDefault + 1;
                if T1 <= T0
                    gui_tools.show_warning('The last historical observation to be displayed must be after the first historical observation.');
                    uicontrol(hObject);
                    return
                elseif T1 > str2double(project_info.nobs)
                    gui_tools.show_warning(['The last historical observation is ' ...
                        num2str(project_info.last_obs_date.time(1)) 'Q' num2str(project_info.last_obs_date.time(2)) '.']);
                    uicontrol(hObject);
                    return
                end
                [ex_names, leg] = get_shock_groups(handles.useShockGrouping.Value);
                gui_shocks.shock_decomp_smooth_q_test([],project_info.first_obs_date(1),ex_names,leg,var_list_,1,[],0,[],[], T0, T1);
            else
                options_.model_settings.shocks = model_settings.shocks;
                options_.shock_grouping = handles.useShockGrouping.Value;
                options_.shock_decomp.init_state = handles.useInitState.Value;
                options_.no_graph.shock_decomposition = handles.noGraph.Value;
                oo_ = shock_decomposition(M_, oo_, options_, var_list_, bayestopt_, estim_params_);
            end
            jObj.stop;
            jObj.setBusyText('All done!');
            project_info.modified = true;
            project_info.shock_decomposition_run = true;
            gui_set_menus(true);
        catch ME
            jObj.stop;
            jObj.setBusyText('Done with errors!');
            gui_tools.show_error('Error in execution of shock decomposition command', ME, 'extended');
            uicontrol(hObject);
        end
        delete(guiObj);
        options_ = old_options;
    end

    function [ex_names, leg] = get_shock_groups(shock_grouping)
        shocks = model_settings.shocks();
        num_shocks = size(shocks,1);
        ex_names = cell(0, num_shocks);
        leg = cell(0, 1);
        num_groups = 0;
        for i = 1:num_shocks
            gname = shocks{i,8};
            sname = shocks{i,1};
            if ~shocks{i,7} %All hidden shocks will be part of Others group
                continue
            end
            if ~shock_grouping
                gname = sname;
            end
            if num_groups==0
                num_groups = 1;
                leg{num_groups,1} = gname;
                ex_names{num_groups,1} = sname;
            else
                ind = find(ismember(char(leg),gname,'rows'));
                if ~isempty(ind)
                    for j = 1:num_shocks
                        if isempty(ex_names{ind,j})
                            ex_names{ind, j} = sname;
                            break
                        end
                    end
                else
                    num_groups = num_groups + 1;
                    leg{num_groups, 1} = gname;
                    ex_names{num_groups, 1} = sname;
                end
            end
        end
        leg{num_groups+1,1} = 'Others';
    end

    function pushbuttonReset_Callback(~, ~)
        handles.endoTable.Data(:, 1) = {false};
        handles.parameterSet.Value = handles.parameterSetDefault;
        handles.firstPeriodQuarter.Value = handles.firstPeriodQuarterDefault;
        handles.firstPeriodYear.Value = handles.firstPeriodYearDefault;
        handles.lastPeriodQuarter.Value = handles.lastPeriodQuarterDefault;
        handles.lastPeriodYear.Value = handles.lastPeriodYearDefault;
        handles.useShockGrouping.Value = 0;
        handles.useInitState.Value = 0;
        handles.type.Value = 1;
        handles.noGraph = 0;
    end

    function close_tab(~, ~, hTab)
        gui_tabs.delete_tab(hTab);
    end

    function parameterSet_CreateFcn(hObject, ~)
        set_background_color(hObject);
        hObject.String = {'calibration','prior_mode', 'mle_mode', 'prior_mean', 'posterior_mode', 'posterior_mean','posterior_median'};

        %Default value: posterior_mean if Metropolis has been run, mle_mode if ML has been conducted, else posterior_mode.
        handles.parameterSetDefault = 5;
        if isfield(oo_, 'posterior_mean')
            handles.parameterSetDefault = 6;
        elseif isfield(oo_, 'mle_mode')
            handles.parameterSetDefault = 3;
        end
        hObject.Value = handles.parameterSetDefault;
    end

    function firstPeriodQuarter_CreateFcn(hObject, ~)
        set_background_color(hObject);
        hObject.String = {'Q1', 'Q2', 'Q3', 'Q4'};
        handles.firstPeriodQuarterDefault = first_period.time(2);
        hObject.Value = handles.firstPeriodQuarterDefault;
    end

    function firstPeriodYear_CreateFcn(hObject, ~)
        set_background_color(hObject);
        hObject.String = cellstr(num2str((first_period.time(1):last_period.time(1))'));
        handles.firstPeriodYearDefault = 1;
        hObject.Value = handles.firstPeriodYearDefault;
    end

    function lastPeriodQuarter_CreateFcn(hObject, ~)
        set_background_color(hObject);
        hObject.String = {'Q1','Q2', 'Q3', 'Q4'};
        handles.lastPeriodQuarterDefault = last_period.time(2);
        hObject.Value = handles.lastPeriodQuarterDefault;
    end

    function lastPeriodYear_CreateFcn(hObject, ~)
        set_background_color(hObject);
        hObject.String = cellstr(num2str((first_period.time(1):last_period.time(1))'));
        handles.lastPeriodYearDefault = last_period.time(1);
        hObject.Value = handles.lastPeriodYearDefault;
    end

    function pushbuttonCloseAll_Callback(~, ~)
        gui_tools.close_all_figures();
    end

    function hObject = set_background_color(hObject)
        if ispc && isequal(hObject.BackgroundColor, get(0, 'defaultUicontrolBackgroundColor'))
            hObject.BackgroundColor = 'white';
        end
    end
end
